@@ -1,10 +1,10 @@
 # -*- coding: utf-8 -*-
-
 # Copyright (C) 2005  Joe Wreschnig
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of version 2 of the GNU General Public License as
-# published by the Free Software Foundation.
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 
 """Read and write FLAC Vorbis comments and stream information.
 
@@ -187,9 +187,11 @@ class StreamInfo(MetadataBlock, mutagen.StreamInfo):
         bits_per_sample (`int`): bits per sample
         total_samples (`int`): total samples in file
         length (`float`): audio length in seconds
+        bitrate (`int`): bitrate in bits per second, as an int
     """
 
     code = 0
+    bitrate = 0
 
     def __eq__(self, other):
         try:
@@ -793,6 +795,14 @@ class FLAC(mutagen.FileType):
         except (AttributeError, IndexError):
             raise FLACNoHeaderError("Stream info block not found")
 
+        if self.info.length:
+            start = fileobj.tell()
+            fileobj.seek(0, 2)
+            self.info.bitrate = int(
+                float(fileobj.tell() - start) * 8 / self.info.length)
+        else:
+            self.info.bitrate = 0
+
     @property
     def info(self):
         return self.metadata_blocks[0]
